package org.infinispan.tutorial.simple.remote;

import java.util.concurrent.TimeUnit;
import java.util.Date;
import java.text.SimpleDateFormat;

import org.infinispan.client.hotrod.RemoteCache;
import org.infinispan.client.hotrod.RemoteCacheManager;
import org.infinispan.client.hotrod.configuration.ConfigurationBuilder;
import org.infinispan.client.hotrod.impl.ConfigurationProperties;
import org.infinispan.client.hotrod.MetadataValue;
import org.infinispan.client.hotrod.Flag;

import org.jboss.logging.Logger;

public class InfinispanRemote {

    private static final Logger LOGGER = Logger.getLogger(InfinispanRemote.class);
    private static final SimpleDateFormat SDF = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss SSS");

    public static void main(String[] args) {

        // Create a configuration for a locally-running server
        ConfigurationBuilder builder = new ConfigurationBuilder();
        builder.addServer().host("127.0.0.1").port(ConfigurationProperties.DEFAULT_HOTROD_PORT);
        // builder.forceReturnValues(true);

        // Connect to the server
        RemoteCacheManager cacheManager = new RemoteCacheManager(builder.build());
        // Obtain the remote cache
        RemoteCache<String, String> cache = cacheManager.getCache();

        try {
            String key = "key";
            String val = "value";
            // String val = Long.toString(System.currentTimeMillis());

            // long lifespanTime = 2147483L; // OK
            long lifespanTime = 2147484L; // KO : treated as -1
            // long lifespanTime = 2147483647L; // Integer.MAX_VALUE
            long maxIdleTime = lifespanTime;

            LOGGER.infof("Invoke put() new entry \"%s=%s\" with lifespan=%s, maxIdle=%s", key, val, lifespanTime, maxIdleTime);
            cache.put(key, val, lifespanTime, TimeUnit.SECONDS, maxIdleTime, TimeUnit.SECONDS);

            LOGGER.infof("New entry created : %s = %s", key, cache.get(key));

            // Retrieve the metadata of the entry
            MetadataValue metadata = cache.getWithMetadata(key);
            long lifespan = metadata.getLifespan();  // the Lifespan if expiration with lifespan isactive for this entry
            long maxIdle = metadata.getMaxIdle();  // lhe max idle time in ms if active for this entry
            long created = metadata.getCreated();  // the creation time (ms since 1970)
            long lastUsed = metadata.getLastUsed();  // the last access time (ms since 1970)
            LOGGER.infof("New entry metadata: lifespan=%s, maxIlde=%s, lastUsed=%s, created=%s", lifespan, maxIdle, lastUsed, SDF.format(new Date(created)));

            LOGGER.infof("Removing the entry: %s = %s", key, cache.get(key));
            cache.remove(key);

        } catch(Exception e) {
            LOGGER.error("Exception occurred", e);
        }

        // Stop the cache manager and release all resources
        cacheManager.stop();
    }

}
